'use strict';

var parallastic = function parallastic() {
  var scrollable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : window;

  var listeners = [];
  var eventTarget = scrollable;
  var propertyProvider = scrollable === window ? document.body : scrollable;

  var handleScroll = function handleScroll(e) {
    var toRemove = [];
    // Give every listener a chance
    for (var i = 0; i < listeners.length; i++) {
      var listener = listeners[i];
      var ret = listener(e);
      if (ret) toRemove.push(listener);
    }
    // Remove the ones which reported they are done
    // (Not removed before because can't remove during for on same array)
    for (var _i = 0; _i < toRemove.length; _i++) {
      var _listener = toRemove[_i];
      removeListener(_listener);
    }
  };

  var addListener = function addListener(func) {
    listeners.push(func);
    if (listeners.length === 1) {
      eventTarget.addEventListener('scroll', handleScroll);
    }
    return func;
  };

  var removeListener = function removeListener(func) {
    var idx = listeners.indexOf(func);
    if (idx >= 0) listeners.splice(idx, 1);
    if (listeners.length === 0) {
      eventTarget.removeEventListener('scroll', handleScroll);
    }
    return func;
  };

  var teardown = function teardown() {
    if (listeners.length > 0) {
      eventTarget.removeEventListener('scroll', handleScroll);
    }
    listeners.splice(0, listeners.length);
  };

  var addToggler = function addToggler(target, className, position) {
    var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    if (!target) {
      console.warn('AddToggler: No target specified.');
      return;
    }
    // Accept previous "reverse" parameter for now
    if (opts === true) opts = { reverse: true };
    var classTarget = opts.classTarget || target;
    // Add the listener and return it so it can be removed
    return addListener(function (e) {
      var delta = target.getBoundingClientRect().top;
      var percent = 1 - delta / propertyProvider.clientHeight;
      if (percent >= position) {
        classTarget.classList.add(className);
        return !opts.reverse;
      } else if (opts.reverse) {
        classTarget.classList.remove(className);
      }
    });
  };

  var addProportional = function addProportional(target, yStart, yEnd, property, propertyStart, propertyEnd) {
    var opts = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};

    if (!target) {
      console.warn('AddProportional: No target specified.');
      return;
    }
    return addListener(function (e) {
      var ref = opts.ref || propertyProvider;
      var thisYStart = yStart;
      var thisYEnd = yEnd;
      if (opts.units === '%') {
        thisYStart *= ref.clientHeight / 100;
        thisYEnd *= ref.clientHeight / 100;
      }
      thisYStart += ref.offsetTop;
      thisYEnd += ref.offsetTop;
      if (opts.ref) thisYStart -= propertyProvider.clientHeight;
      var scrollTop = propertyProvider.scrollTop || window.scrollY;
      var percent = (scrollTop - thisYStart) / (thisYEnd - thisYStart);
      if (percent < 0) percent = 0;
      if (percent > 1) percent = 1;
      var value = propertyStart + percent * (propertyEnd - propertyStart);
      if (opts.prefix) value = opts.prefix + value;
      if (opts.suffix) value += opts.suffix;
      target.style[property] = value;
    });
  };

  return {
    addToggler: addToggler,
    addProportional: addProportional,
    removeListener: removeListener,
    forceUpdate: handleScroll,
    teardown: teardown
  };
};

module.exports = parallastic;